package com.bergerkiller.bukkit.nolagg;

import java.util.LinkedList;
import java.util.logging.Level;

import org.bukkit.Bukkit;

public class OperationScheduler extends Thread {
	
	private static LinkedList<ThreadOperation> operations = new LinkedList<ThreadOperation>();
	private static LinkedList<ThreadOperation> finished = new LinkedList<ThreadOperation>();
	private static boolean running = false;
	private static boolean ended = false;
	private static int endtaskid = -1;

	public static void start(ThreadOperation operation) {
		synchronized (operations) {
			operations.offer(operation);
		}
		operation.start();
	}
	
	public static void init() {
		running = true;
		new OperationScheduler().start();
		endtaskid = Bukkit.getServer().getScheduler().scheduleSyncRepeatingTask(NoLagg.plugin, new Runnable() {
			public void run() {
				synchronized (finished) {
					ThreadOperation op;
					while ((op = finished.poll()) != null) {
						try {
						    op.end();
						} catch (Throwable t) {
							NoLagg.log(Level.SEVERE, "An error occured while processing scheduled operations:");
							t.printStackTrace();
						}
					}
				}
			}
		}, 1, 1);
	}
	public static void deinit() {
		if (endtaskid != -1) {
			Bukkit.getServer().getScheduler().cancelTask(endtaskid);
		}
		running = false;
		//wait for it to be finished
		synchronized (operations) {
			if (!operations.isEmpty()) {
				NoLagg.log(Level.INFO, "Performing remaining scheduled operations (" + operations.size() + ") ...");
			}
			while (!ended) {
				try {
					Thread.sleep(10);
				} catch (InterruptedException ex) {}
			}
			for (ThreadOperation op : operations) {
				try {
				    op.run();
				    op.end();
				} catch (Throwable t) {
					NoLagg.log(Level.SEVERE, "An error occured while processing scheduled operations:");
					t.printStackTrace();
				}
			}
			operations.clear();
		}
		synchronized (finished) {
			for (ThreadOperation op : finished) {
				try {
				    op.end();
				} catch (Throwable t) {
					NoLagg.log(Level.SEVERE, "An error occured while processing scheduled operations:");
					t.printStackTrace();
				}
			}
			finished.clear();
		}
	}
	
	public void run() {
		ThreadOperation operation;
		while (!this.isInterrupted() && running) {
			synchronized (operations) {
				operation = operations.poll();
			}
			if (operation == null) {
				try {
					Thread.sleep(300);
				} catch (InterruptedException ex) {}
			} else {
				try {
					operation.run();
					synchronized (finished) {
						finished.offer(operation);
					}
				} catch (Throwable t) {
					NoLagg.log(Level.SEVERE, "An error occured while processing scheduled operations:");
					t.printStackTrace();
				}
			}
		}
		ended = true;
	}

}
